"ToolDelta 常量定义"

PLUGIN_MARKET_SOURCE_OFFICIAL = (
    "https://tdload.whiteleaf.cn"
    "https://raw.githubusercontent.com/ToolDelta/ToolDelta-PluginMarket/main"
)
"插件市场源"

LAUNCH_CFG: dict = {
    "启动器启动模式(请不要手动更改此项, 改为0可重置)": 0,
    "验证服务器地址(更换时记得更改fbtoken)": "",
    "是否记录日志": True,
    "是否使用github镜像": True,
    "插件市场源": PLUGIN_MARKET_SOURCE_OFFICIAL,
}
"默认登录配置"

LAUNCH_CFG_STD: dict = {
    "启动器启动模式(请不要手动更改此项, 改为0可重置)": int,
    "验证服务器地址(更换时记得更改fbtoken)": str,
    "是否记录日志": bool,
    "是否使用github镜像": bool,
    "插件市场源": str,
}
"默认登录配置标准验证格式"

LAUNCHER_NEOMEGA_STD: dict = {
    "服务器号": int,
    "密码": str,
    "验证服务器地址(更换时记得更改fbtoken)": str,
}
"启动器：NeOmega 启动配置验证格式"

LAUNCHER_NEOMEGA_DEFAULT: dict = {
    "服务器号": 0,
    "密码": "",
    "验证服务器地址(更换时记得更改fbtoken)": "",
}
LAUNCHER_NEOMGPARALLELTTOOLDELTA_DEFAULT: dict = {
    "服务器号": 0,
    "密码": "",
    "验证服务器地址(更换时记得更改fbtoken)": "",
}
LAUNCHER_NEOMGPARALLELTTOOLDELTA_STD: dict = {
    "服务器号": int,
    "密码": str,
    "验证服务器地址(更换时记得更改fbtoken)": str,
}

"启动器：NeOmega 默认启动配置"

LAUNCHER_BEWS_STD: dict = {"服务端开放地址": str}
"启动器：BEWSServer 启动配置验证格式"

LAUNCHER_BEWS_DEFAULT: dict = {"服务端开放地址": ""}
"启动器：BEWSServer 默认启动配置"

FB_APIS = [
    "https://user.fastbuilder.pro/api/phoenix/login",
    "https://user.fastbuilder.pro/api/new",
    "https://user.fastbuilder.pro",
]
"验证服务器：FastBuilder API 列表"


GUGU_APIS = [
    "https://liliya233.uk/api/phoenix/login",
    "https://liliya233.uk/api/new",
    "https://liliya233.uk",
]
"验证服务器：Liliya API 列表"

AUTH_SERVERS = [
    ("FastBuilder 官方验证服务器", "https://user.fastbuilder.pro"),
    ("咕咕酱 FB验证服务器", "https://liliya233.uk"),
]
"验证服务器列表"

TOOLDELTA_PLUGIN_DIR = "插件文件"
"插件文件路径"

TOOLDELTA_CLASSIC_PLUGIN = "ToolDelta类式插件"
"插件文件：ToolDelta 类式插件 路径"

TOOLDELTA_INJECTED_PLUGIN = "ToolDelta注入式插件"
"插件文件：ToolDelta 注入式插件 路径"

TOOLDELTA_PLUGIN_DATA_DIR = "插件数据文件"
"插件数据文件文件夹路径"

PLUGIN_TYPE_MAPPING = {
    "classic": TOOLDELTA_CLASSIC_PLUGIN,
    "injected": TOOLDELTA_INJECTED_PLUGIN,
}
"插件属性名映射"

TDSPECIFIC_MIRROR = "https://tdload.whiteleaf.cn"
"ToolDelta专用镜像"


class PacketIDS:
    "数据包 ID 常量表"

    IDLogin = 1  # 客户端登录（本地）
    IDPlayStatus = 2  # 玩家状态
    IDServerToClientHandshake = 3  # 服务端到客户端握手
    IDClientToServerHandshake = 4  # 客户端到服务端握手
    IDDisconnect = 5  # 断开连接
    IDResourcePacksInfo = 6  # 资源包信息（非常用）
    IDResourcePackStack = 7  # 资源包堆叠（非常用）
    IDResourcePackClientResponse = 8  # 资源包客户端响应（非常用）
    IDText = 9  # 文本消息
    IDSetTime = 10  # 更新客户端时间（服务端 -> 客户端）
    IDStartGame = 11  # 开始游戏
    IDAddPlayer = 112  # 添加玩家实体
    IDAddActor = 13  # 添加实体
    IDRemoveActor = 14  # 添加实体
    IDAddItemActor = 15  # 添加物品实体
    IDTakeItemActor = 17  # 捡起物品实体（动画）
    IDMoveActorAbsolute = 18  # 移动实体到绝对位置
    IDMovePlayer = 19  # 玩家移动（服务端 <-> 客户端）
    IDPassengerJump = 20  # 乘骑跳跃（客户端 -> 服务端）
    IDUpdateBlock = 21  # 更新方块（单方块修改）
    IDAddPainting = 22  # 添加绘画实体
    IDTickSync = 23  # 同步Tick（服务端 <-> 客户端）
    IDLevelSoundEventV1 = 24  # ???
    IDLevelEvent = 25  # 世界事件（服务端 -> 客户端）
    IDBlockEvent = 26  # 方块事件（服务端 -> 客户端）[打开箱子./././...]
    IDActorEvent = 27  # 实体事件（服务端 -> 客户端）[狼抖干自己./././...]
    IDMobEffect = 28  # 生物效果（服务端 -> 客户端）
    IDUpdateAttributes = 29  # 更新实体属性（服务端 -> 客户端）[移动速度./健康状况./...]
    IDInventoryTransaction = 30  # 物品交易（服务端 <- 客户端）
    IDMobEquipment = 31  # 实体物品持有（服务端 <-> 客户端）[僵尸手持石剑././...]
    IDMobArmourEquipment = 32  # 装备穿戴（服务端 -> 客户端）[玩家./僵尸./其他实体./...]
    IDInteract = 33  # 实体交互（弃用）
    IDBlockPickRequest = 34  # 拾取物品请求（客户端 -> 服务端）
    IDActorPickRequest = 35  # 拾取实体请求（客户端 -> 服务端）
    IDPlayerAction = 36  # 玩家行为（客户端 -> 服务端）
    IDHurtArmour = 38  # 盔甲损害（服务端 -> 客户端）
    IDSetActorData = 39  # 实体元数据（服务端 -> 客户端）[实体是否着火././...]
    IDSetActorMotion = 40  # 设置客户端速度（服务端 -> 客户端）
    IDSetActorLink = 41  # 设置实体乘骑（服务端 -> 客户端）
    IDSetHealth = 42  # 设置玩家血量（服务端 -> 客户端）
    IDSetSpawnPosition = 43  # 设置玩家出生点位置（服务端 -> 客户端）
    IDAnimate = 44  # 动画效果（服务端 -> 客户端）
    IDRespawn = 45  # 重生（服务端 <-> 客户端）
    IDContainerOpen = 46  # 打开容器（服务端 -> 客户端）
    IDContainerClose = 47  # 关闭容器（服务端 -> 客户端）
    IDPlayerHotBar = 48  # 玩家快捷栏槽位（服务端 -> 客户端）
    IDInventoryContent = 49  # 更新玩家背包（服务端 -> 客户端）
    IDInventorySlot = 50  # 玩家背包单槽位更新（服务端 -> 客户端）
    IDContainerSetData = 51  # 容器设置数据（服务端 -> 客户端）
    IDCraftingData = 52  # 合成数据（服务端 -> 客户端）
    IDCraftingEvent = 53  # 合成事件（客户端 -> 服务端）
    IDGUIDataPickItem = 54  # GUI数据拾取物品（客户端 -> 服务端）
    IDAdventureSettings = 55  # 冒险设置（服务端 -> 客户端）
    IDBlockActorData = 56  # 方块实体数据（服务端 -> 客户端）
    IDPlayerInput = 57  # 玩家输入（客户端 -> 服务端）
    IDLevelChunk = 58  # 区块数据（服务端 -> 客户端）
    IDSetCommandsEnabled = 59  # 设置命令启用（服务端 -> 客户端）
    IDSetDifficulty = 60  # 设置难度（服务端 -> 客户端）
    IDChangeDimension = 61  # 改变维度（服务端 -> 客户端）
    IDSetPlayerGameType = 62  # 设置玩家游戏类型（服务端 -> 客户端）
    IDPlayerList = 63  # 玩家列表（服务端 -> 客户端）
    IDSimpleEvent = 64  # 简单事件（服务端 -> 客户端）
    IDEvent = 65  # 事件（服务端 -> 客户端）
    IDSpawnExperienceOrb = 66  # 生成经验球（服务端 -> 客户端）
    IDClientBoundMapItemData = 67  # 客户端绑定地图物品数据（服务端 -> 客户端）
    IDMapInfoRequest = 68  # 地图信息请求（客户端 -> 服务端）
    IDRequestChunkRadius = 69  # 请求区块半径（客户端 -> 服务端）
    IDChunkRadiusUpdated = 70  # 区块半径更新（服务端 -> 客户端）
    IDItemFrameDropItem = 71  # 物品展示框掉落物品（服务端 -> 客户端）
    IDGameRulesChanged = 72  # 游戏规则改变（服务端 -> 客户端）
    IDCamera = 73  # 相机（服务端 -> 客户端）
    IDBossEvent = 74  # Boss事件（服务端 -> 客户端）
    IDShowCredits = 75  # 显示 credits（服务端 -> 客户端）
    IDAvailableCommands = 76  # 可用命令（服务端 -> 客户端）
    IDCommandRequest = 77  # 命令请求（客户端 -> 服务端）
    IDCommandBlockUpdate = 78  # 命令块更新（客户端 -> 服务端）
    IDCommandOutput = 79  # 命令输出（服务端 -> 客户端）
    IDUpdateTrade = 80  # 更新交易（服务端 -> 客户端）
    IDUpdateEquip = 81  # 更新装备（服务端 -> 客户端）
    IDResourcePackDataInfo = 82  # 资源包数据信息（服务端 -> 客户端）
    IDResourcePackChunkData = 83  # 资源包块数据（服务端 -> 客户端）
    IDResourcePackChunkRequest = 84  # 资源包块请求（客户端 -> 服务端）
    IDTransfer = 85  # 传输（服务端 -> 客户端）
    IDPlaySound = 86  # 播放声音（服务端 -> 客户端）
    IDStopSound = 87  # 停止声音（服务端 -> 客户端）
    IDSetTitle = 88  # 设置标题（服务端 -> 客户端）
    IDAddBehaviourTree = 89  # 添加行为树（服务端 -> 客户端）
    IDStructureBlockUpdate = 90  # 结构方块更新（客户端 -> 服务端）
    IDShowStoreOffer = 91  # 显示商店优惠（服务端 -> 客户端）
    IDPurchaseReceipt = 92  # 购买收据（客户端 -> 服务端）
    IDPlayerSkin = 93  # 玩家皮肤（客户端 -> 服务端）
    IDSubClientLogin = 94  # 子客户端登录（客户端 -> 服务端）
    IDAutomationClientConnect = 95  # 自动化客户端连接（客户端 -> 服务端）
    IDSetLastHurtBy = 96  # 设置最后受伤来源（服务端 -> 客户端）
    IDBookEdit = 97  # 书本编辑（客户端 -> 服务端）
    IDNPCRequest = 98  # NPC请求（客户端 -> 服务端）
    IDPhotoTransfer = 99  # 照片传输（客户端 -> 服务端）
    IDModalFormRequest = 100  # 模态表单请求（服务端 -> 客户端）
    IDModalFormResponse = 101  # 模态表单响应（客户端 -> 服务端）
    IDServerSettingsRequest = 102  # 服务器设置请求（客户端 -> 服务端）
    IDServerSettingsResponse = 103  # 服务器设置响应（服务端 -> 客户端）
    IDShowProfile = 104  # 显示资料（客户端 -> 服务端）
    IDSetDefaultGameType = 105  # 设置默认游戏类型（服务端 -> 客户端）
    IDRemoveObjective = 106  # 移除目标（服务端 -> 客户端）
    IDSetDisplayObjective = 107  # 设置显示目标（服务端 -> 客户端）
    IDSetScore = 108  # 设置分数（服务端 -> 客户端）
    IDLabTable = 109  # 实验室表（服务端 -> 客户端）
    IDUpdateBlockSynced = 110  # 同步更新方块（服务端 -> 客户端）
    IDMoveActorDelta = 111  # 移动实体增量（服务端 -> 客户端）
    IDSetScoreboardIdentity = 112  # 设置计分板身份（服务端 -> 客户端）
    IDSetLocalPlayerAsInitialised = 113  # 设置本地玩家为已初始化（服务端 -> 客户端）
    IDUpdateSoftEnum = 114  # 更新软枚举（服务端 -> 客户端）
    IDNetworkStackLatency = 115  # 网络堆栈延迟（服务端 -> 客户端）
    IDScriptCustomEvent = 117  # 脚本自定义事件（客户端 -> 服务端）
    IDSpawnParticleEffect = 118  # 生成粒子效果（服务端 -> 客户端）
    IDAvailableActorIdentifiers = 119  # 可用实体标识符（服务端 -> 客户端）
    IDLevelSoundEventV2 = 120  # 世界声音事件 V2（服务端 -> 客户端）
    IDNetworkChunkPublisherUpdate = 121  # 网络区块发布者更新（服务端 -> 客户端）
    IDBiomeDefinitionList = 122  # 生物群系定义列表（服务端 -> 客户端）
    IDLevelSoundEvent = 123  # 世界声音事件（服务端 -> 客户端）
    IDLevelEventGeneric = 124  # 世界事件通用（服务端 -> 客户端）
    IDLecternUpdate = 125  # 讲台更新（客户端 -> 服务端）
    IDAddEntity = 127  # 添加实体（服务端 -> 客户端）
    IDRemoveEntity = 128  # 移除实体（服务端 -> 客户端）
    IDClientCacheStatus = 129  # 客户端缓存状态（客户端 -> 服务端）
    IDOnScreenTextureAnimation = 130  # 屏幕纹理动画（服务端 -> 客户端）
    IDMapCreateLockedCopy = 131  # 地图创建锁定副本（服务端 -> 客户端）
    IDStructureTemplateDataRequest = 132  # 结构模板数据请求（客户端 -> 服务端）
    IDStructureTemplateDataResponse = 133  # 结构模板数据响应（服务端 -> 客户端）
    IDClientCacheBlobStatus = 135  # 客户端缓存块状态（客户端 -> 服务端）
    IDClientCacheMissResponse = 136  # 客户端缓存未命中响应（服务端 -> 客户端）
    IDEducationSettings = 137  # 教育设置（服务端 -> 客户端）
    IDEmote = 138  # 表情（客户端 -> 服务端）
    IDMultiPlayerSettings = 139  # 多人游戏设置（客户端 -> 服务端）
    IDSettingsCommand = 140  # 设置命令（客户端 -> 服务端）
    IDAnvilDamage = 141  # 铁砧损坏（服务端 -> 客户端）
    IDCompletedUsingItem = 142  # 完成使用物品（客户端 -> 服务端）
    IDNetworkSettings = 143  # 网络设置（服务端 -> 客户端）
    IDPlayerAuthInput = 144  # 玩家认证输入（客户端 -> 服务端）
    IDCreativeContent = 145  # 创造内容（服务端 -> 客户端）
    IDPlayerEnchantOptions = 146  # 玩家附魔选项（服务端 -> 客户端）
    IDItemStackRequest = 147  # 物品堆栈请求（客户端 -> 服务端）
    IDItemStackResponse = 148  # 物品堆栈响应（服务端 -> 客户端）
    IDPlayerArmourDamage = 149  # 玩家盔甲损坏（服务端 -> 客户端）
    IDCodeBuilder = 150  # 代码构建器（客户端 -> 服务端）
    IDUpdatePlayerGameType = 151  # 更新玩家游戏类型（服务端 -> 客户端）
    IDEmoteList = 152  # 表情列表（服务端 -> 客户端）
    IDPositionTrackingDBServerBroadcast = (
        153  # 位置跟踪数据库服务器广播（服务端 -> 客户端）
    )
    IDPositionTrackingDBClientRequest = (
        154  # 位置跟踪数据库客户端请求（客户端 -> 服务端）
    )
    IDDebugInfo = 155  # 调试信息（服务端 -> 客户端）
    IDPacketViolationWarning = 156  # 数据包违规警告（服务端 -> 客户端）
    IDMotionPredictionHints = 157  # 运动预测提示（服务端 -> 客户端）
    IDAnimateEntity = 158  # 动画实体（服务端 -> 客户端）
    IDCameraShake = 159  # 相机震动（服务端 -> 客户端）
    IDPlayerFog = 160  # 玩家迷雾（服务端 -> 客户端）
    IDCorrectPlayerMovePrediction = 161  # 纠正玩家移动预测（服务端 -> 客户端）
    IDItemComponent = 162  # 物品组件（服务端 -> 客户端）
    IDFilterText = 163  # 过滤文本（客户端 -> 服务端）
    IDClientBoundDebugRenderer = 164  # 客户端绑定调试渲染器（服务端 -> 客户端）
    IDSyncActorProperty = 165  # 同步实体属性（服务端 -> 客户端）
    IDAddVolumeEntity = 166  # 添加体积实体（服务端 -> 客户端）
    IDRemoveVolumeEntity = 167  # 移除体积实体（服务端 -> 客户端）
    IDSimulationType = 168  # 模拟类型（服务端 -> 客户端）
    IDNPCDialogue = 169  # NPC对话（服务端 -> 客户端）
    IDEducationResourceURI = 170  # 教育资源URI（服务端 -> 客户端）
    IDCreatePhoto = 171  # 创建照片（服务端 -> 客户端）
    IDUpdateSubChunkBlocks = 172  # 更新子区块方块（服务端 -> 客户端）
    IDPhotoInfoRequest = 173  # 照片信息请求（客户端 -> 服务端）
    IDSubChunk = 174  # 子区块（服务端 -> 客户端）
    IDSubChunkRequest = 175  # 子区块请求（客户端 -> 服务端）
    IDClientStartItemCooldown = 176  # 客户端开始物品冷却（客户端 -> 服务端）
    IDScriptMessage = 177  # 脚本消息（客户端 -> 服务端）
    IDCodeBuilderSource = 178  # 代码构建器源（客户端 -> 服务端）
    IDTickingAreasLoadStatus = 179  #  ticking 区域加载状态（服务端 -> 客户端）
    IDDimensionData = 180  # 维度数据（服务端 -> 客户端）
    IDAgentAction = 181  # 代理
    IDChangeMobProperty = 182  # 改变生物属性（服务端 -> 客户端）
    IDLessonProgress = 183  # 课程进度（客户端 -> 服务端）
    IDRequestAbility = 184  # 请求能力（客户端 -> 服务端）
    IDRequestPermissions = 185  # 请求权限（客户端 -> 服务端）
    IDToastRequest = 186  # 吐司请求（服务端 -> 客户端）
    IDUpdateAbilities = 187  # 更新能力（服务端 -> 客户端）
    IDUpdateAdventureSettings = 188  # 更新冒险设置（服务端 -> 客户端）
    IDDeathInfo = 189  # 死亡信息（服务端 -> 客户端）
    IDEditorNetwork = 190  # 编辑器网络（服务端 -> 客户端）
    IDFeatureRegistry = 191  # 特性注册表（服务端 -> 客户端）
    IDServerStats = 192  # 服务器统计（服务端 -> 客户端）
    IDRequestNetworkSettings = 193  # 请求网络设置（客户端 -> 服务端）
    IDGameTestRequest = 194  # 游戏测试请求（客户端 -> 服务端）
    IDGameTestResults = 195  # 游戏测试结果（服务端 -> 客户端）
    IDUpdateClientInputLocks = 196  # 更新客户端输入锁（服务端 -> 客户端）
    IDClientCheatAbility = 197  # 客户端作弊能力（客户端 -> 服务端）
    IDCameraPresets = 198  # 相机预设（服务端 -> 客户端）
    IDUnlockedRecipes = 199  # 已解锁配方（服务端 -> 客户端）
    IDPyRpc = 200  # Python远程过程调用（客户端 -> 服务端）
    IDChangeModel = 201  # 改变模型（服务端 -> 客户端）
    IDStoreBuySucc = 202  # 商店购买成功（服务端 -> 客户端）
    IDNeteaseJson = 203  # 网易 JSON（客户端 -> 服务端）
    IDChangeModelTexture = 204  # 改变模型纹理（服务端 -> 客户端）
    IDChangeModelOffset = 205  # 改变模型偏移（服务端 -> 客户端）
    IDChangeModelBind = 206  # 改变模型绑定（服务端 -> 客户端）
    IDHungerAttr = 207  # 饥饿属性（服务端 -> 客户端）
    IDSetDimensionLocalTime = 208  # 设置维度本地时间（服务端 -> 客户端）
    IDWithdrawFurnaceXp = 209  # 提取熔炉经验（客户端 -> 服务端）
    IDSetDimensionLocalWeather = 210  # 设置维度本地天气（服务端 -> 客户端）
    IDCustomV1 = 223  # 自定义 V1（服务端 -> 客户端）
    IDCombine = 224  # 组合（服务端 -> 客户端）
    IDVConnection = 225  # V 连接（服务端 -> 客户端）
    IDTransport = 226  # 传输（服务端 -> 客户端）
    IDCustomV2 = 227  # 自定义 V2（服务端 -> 客户端）
    IDConfirmSkin = 228  # 确认皮肤（服务端 -> 客户端）
    IDTransportNoCompress = 229  # 无压缩传输（服务端 -> 客户端）
    IDMobEffectV2 = 230  # 生物效果 V2（服务端 -> 客户端）
    IDMobBlockActorChanged = 231  # 生物方块实体改变（服务端 -> 客户端）
    IDChangeActorMotion = 232  # 改变实体运动（服务端 -> 客户端）
    IDAnimateEmoteEntity = 233  # 动画表情实体（服务端 -> 客户端）
    IDCameraInstruction = 300  # 相机指令（服务端 -> 客户端）
    IDCompressedBiomeDefinitionList = 301  # 压缩生物群系定义列表（服务端 -> 客户端）
    IDTrimData = 302  # 修剪数据（服务端 -> 客户端）
    IDOpenSign = 303  # 打开标志（服务端 -> 客户端）
    IDAgentAnimation = 304  # 代理动画（服务端 -> 客户端）


class ContainerType:
    ContainerTypeInventory = -1  # 物品栏
    ContainerTypeContainer = 0  # 容器
    ContainerTypeWorkbench = 1  # 工作台
    ContainerTypeFurnace = 2  # 熔炉
    ContainerTypeEnchantment = 3  # 附魔台
    ContainerTypeBrewingStand = 4  # 酿造台
    ContainerTypeAnvil = 5  # 铁砧
    ContainerTypeDispenser = 6  # 发射器
    ContainerTypeDropper = 7  # 投掷器
    ContainerTypeHopper = 8  # 漏斗
    ContainerTypeCauldron = 9  # 炼药锅
    ContainerTypeCartChest = 10  # 箱子矿车
    ContainerTypeCartHopper = 11  # 漏斗矿车
    ContainerTypeHorse = 12  # 马
    ContainerTypeBeacon = 13  # 信标
    ContainerTypeStructureEditor = 14  # 结构编辑器
    ContainerTypeTrade = 15  # 交易
    ContainerTypeCommandBlock = 16  # 命令方块
    ContainerTypeJukebox = 17  # 唱片机
    ContainerTypeArmour = 18  # 盔甲
    ContainerTypeHand = 19  # 手持
    ContainerTypeCompoundCreator = 20  # 化合物创建器
    ContainerTypeElementConstructor = 21  # 元素构造器
    ContainerTypeMaterialReducer = 22  # 材料分解器
    ContainerTypeLabTable = 23  # 实验台
    ContainerTypeLoom = 24  # 织布机
    ContainerTypeLectern = 25  # 讲台
    ContainerTypeGrindstone = 26  # 磨石
    ContainerTypeBlastFurnace = 27  # 高炉
    ContainerTypeSmoker = 28  # 烟熏炉
    ContainerTypeStonecutter = 29  # 切石机
    ContainerTypeCartography = 30  # 制图台
    ContainerTypeHUD = 31  # HUD
    ContainerTypeJigsawEditor = 32  # 拼图编辑器
    ContainerTypeSmithingTable = 33  # 锻造台
    ContainerTypeChestBoat = 34  # 箱子船
    ContainerTypeDecoratedPot = 35  # 装饰罐
    ContainerTypeCrafter = 36  # 工匠


class PlayerActionType:
    PlayerActionStartBreak = 0
    PlayerActionAbortBreak = 1
    PlayerActionStopBreak = 2
    PlayerActionGetUpdatedBlock = 3
    PlayerActionDropItem = 4
    PlayerActionStartSleeping = 5
    PlayerActionStopSleeping = 6
    PlayerActionRespawn = 7
    PlayerActionJump = 8
    PlayerActionStartSprint = 9
    PlayerActionStopSprint = 10
    PlayerActionStartSneak = 11
    PlayerActionStopSneak = 12
    PlayerActionCreativePlayerDestroyBlock = 13
    PlayerActionDimensionChangeDone = 14
    PlayerActionStartGlide = 15
    PlayerActionStopGlide = 16
    PlayerActionBuildDenied = 17
    PlayerActionCrackBreak = 18
    PlayerActionChangeSkin = 19
    PlayerActionSetEnchantmentSeed = 20
    PlayerActionStartSwimming = 21
    PlayerActionStopSwimming = 22
    PlayerActionStartSpinAttack = 23
    PlayerActionStopSpinAttack = 24
    PlayerActionStartBuildingBlock = 25
    PlayerActionPredictDestroyBlock = 26
    PlayerActionContinueDestroyBlock = 27
    PlayerActionStartItemUseOn = 28
    PlayerActionStopItemUseOn = 29
    PlayerActionHandledTeleport = 30
    PlayerActionMissedSwing = 31
    PlayerActionStartCrawling = 32
    PlayerActionStopCrawling = 33


class EffectIDS:
    EffectSpeed = "speed"  # 速度
    EffectSlowNess = "slowness"  # 缓慢
    EffectHaste = "haste"  # 急迫
    EffectMiningFatigue = "mining_fatigue"  # 挖掘疲劳
    EffectStrength = "strength"  # 力量
    EffectInstantHealth = "instant_health"  # 瞬间治疗
    EffectInstantDamage = "instant_damage"  # 瞬间伤害
    EffectJumpBoost = "jump_boost"  # 跳跃提升
    EffectNausea = "nausea"  # 反胃
    EffectRegenerAtion = "regeneration"  # 生命恢复
    EffectResistance = "resistance"  # 耐力
    EffectFireResistance = "fire_resistance"  # 火焰抗性
    EffectWaterBreathing = "water_breathing"  # 水下呼吸
    EffectInvisibility = "invisibility"  # 隐身
    EffectBlindness = "blindness"  # 致盲
    EffectNightVision = "night_vision"  # 夜视
    EffectHunger = "hunger"  # 饥饿
    EffectWeakness = "weakness"  # 虚弱
    EffectPoison = "poison"  # 中毒
    EffectWither = "wither"  # 凋零
    EffectHealthBoost = "health_boost"  # 生命提升
    EffectAbsorption = "absorption"  # 吸收
    EffectSaturation = "saturation"  # 饱和
    EffectLevitation = "levitation"  # 漂浮
    EffectFatalPoison = "fatal_poison"  # 中毒(致命)
    EffectConduitPower = "conduit_power"  # 潮涌能量
    EffectSlowFalling = "slow_falling"  # 缓慢落下
    EffectBadOmen = "bad_omen"  # 不详之兆
    EffectVillageHero = "village_hero"  # 村庄英雄
    EffectDarkNess = "darkness"  # 黑暗


class BuildStructureIDS:
    StructureBuriedTreasure = "buried_treasure"  # 埋藏宝藏
    StructureEndcity = "endcity"  # 末地城
    StructureFortress = "fortress"  # 下界要塞
    StructureMansion = "mansion"  # 林地府邸
    StructureMineShaft = "mineshaft"  # 废弃矿井
    StructureMonuMent = "monument"  # 海底遗迹
    StructurePillagerOutpost = "pillager_outpost"  # 掠夺者前哨站
    StructureRuins = "ruins"  # 水下遗迹
    StructureShipWreck = "shipwreck"  # 沉船
    StructureStrongHold = "stronghold"  # 要塞
    StructureTemple = "temple"  # 沙漠神殿/丛林神殿/沼泽小屋/雪屋
    StructureVillage = "village"  # 村庄


class ParticleIDS:
    ParticleArrowSpellEmitter = "arrow_spell_emitter"  # 从药箭上飘散的药水粒子
    ParticleBasicBubbleParticle = "basic_bubble_particle"  # 实体在水下产生的飞溅粒子，投射物在水下产生的轨迹粒子。该粒子只能在水中生成和存在。
    ParticleBasicBubbleParticleManual = "basic_bubble_particle_manual"  # 漂浮在水面上的实体产生的粒子，水下TNT被点燃时的气泡粒子。该粒子只能在水中生成和存在。
    ParticleBasicCritParticle = "basic_crit_particle"  # 用蓄力完成的弓或装填完成的弩射出的箭产生的轨迹粒子，尖牙进行攻击产生的粒子
    ParticleBasicFlameParticle = "basic_flame_particle"  # 火把；烧制物品的熔炉、高炉和烟熏炉；岩浆怪着地；刷怪笼工作
    ParticleBasicPortalParticle = "basic_portal_particle"  # 末影之眼的轨迹粒子
    ParticleBasicSmokeParticle = "basic_smoke_particle"  # 雨中的熔岩、岩浆块产生的粒子；伴随着lava_particle粒子产生的粒子；火、火把、灵魂火把的烟雾粒子；投掷器投掷物品；发射器发射物品；烈焰人；末地传送门；酿造台；正在工作的刷怪笼；烧制物品的熔炉、高炉、烟熏炉；点燃的蜡烛；凋灵；营火；凋零玫瑰；驯服生物失败；给棕色哞菇喂食小型花；使用末影之眼；试炼刷怪笼、宝库、实验台产出废物、制作加热块[仅教育版]；凋灵之首、火球的飞行轨迹粒子
    ParticleBlockDestruct = "block_destruct"  # 方块被摧毁；生物摔伤；玩家疾跑；羊吃草；兔子跳跃；嗅探兽刨挖；盔甲架被玩家破坏；使用刷子；实验台制作超级肥料[仅教育版]时的粒子；旋风人时的方块碎片粒子
    ParticleBlockSlide = "block_slide"  # 从蜂蜜块的侧面滑下的粒子
    ParticleBlueFlameParticle = "blue_flame_particle"  # 灵魂火把火苗粒子
    ParticleBreakingItemIcon = "breaking_item_icon"  # 玩家进食；狐狸进食；鸡蛋、喷溅药水、滞留药水、附魔之瓶、雪球和冰弹[仅教育版]破碎；史莱姆跳跃；海龟蛋孵化时的物品碎片粒子
    ParticleBreezeGroundParticle = "breeze_ground_particle"  # 旋风人产生的方块碎片粒子
    ParticleBreezeWindExplosionEmitter = "breeze_wind_explosion_emitter"  # 风弹爆炸粒子
    ParticleBubbleColumnBubble = "bubble_column_bubble"  # 气泡柱顶部的实体产生的气泡粒子。该粒子只能在水中生成和存在。
    # ------------------------------------> 待完成 <------------------------------------



"插件市场客户端"

import asyncio
import os
import platform
import shlex
import time
import traceback

import requests
import ujson as json

from . import urlmethod
from .cfg import Cfg
from .color_print import Print
from .constants import (
    PLUGIN_MARKET_SOURCE_OFFICIAL,
    TOOLDELTA_CLASSIC_PLUGIN,
    TOOLDELTA_INJECTED_PLUGIN,
    LAUNCH_CFG
)
from .plugin_load import PluginRegData
from .plugin_load.PluginGroup import plugin_group
from .utils import Utils

if platform.system().lower() == "windows":
    CLS_CMD = "cls"
else:
    CLS_CMD = "clear"


def clear_screen() -> None:
    "清屏"
    os.system(shlex.quote(CLS_CMD))

def url_join(*urls) -> str:
    """连接 URL

    Returns:
        str: 连接后的 URL
    """
    return "/".join(urls)


def get_json_from_url(url: str) -> dict:
    """从 URL 获取 JSON 数据

    Args:
        url (str): URL

    Raises:
        requests.RequestException: Url 请求失败
        requests.RequestException: 服务器返回了不正确的答复

    Returns:
        dict: JSON 数据
    """
    try:
        resp = requests.get(url, timeout=5)
        resp.raise_for_status()
        return resp.json()
    except requests.RequestException as exc:
        raise requests.RequestException("URL 请求失败") from exc
    except json.JSONDecodeError as exc:
        raise requests.RequestException(
            f"服务器返回了不正确的答复：{resp.text}"
        ) from exc


class PluginMarket:
    "插件市场类"

    def __init__(self):
        self.plugin_id_name_map: dict | None = None
        try:
            self.plugins_download_url = Cfg().get_cfg(
                "ToolDelta基本配置.json", {"插件市场源": str}
            )["插件市场源"]
        except Exception:
            self.plugins_download_url = PLUGIN_MARKET_SOURCE_OFFICIAL

    def enter_plugin_market(self, source_url: str | None = None, in_game=False) -> None:
        """进入插件市场

        Args:
            source_url (str | None, optional): 插件市场源
            in_game (bool, optional): 是否在游戏内
        """
        Print.clean_print("§6正在连接到插件市场..")
        self.plugin_id_name_map = self.get_plugin_id_name_map()
        CTXS = 12

        def display_plugins(start_index: int, total_pages: int):
            """显示插件列表

            Args:
                start_index (int): 起始索引
                total_pages (int): 总页数
            """
            clear_screen()
            Print.print_inf(
                f"{market_datas['SourceName']}: {market_datas['Greetings']}",
                need_log=False,
            )
            for i in range(start_index, min(start_index + CTXS, all_indexes)):
                plugin_id = plugins_list[i][0]
                plugin_name = plugin_ids_map[plugin_id]
                plugin_basic_datas = plugins_list[i][1]
                plugin_type = {"classic": "类式", "injected": "注入式"}.get(
                    plugin_basic_datas.get("plugin-type", "unknown"),
                    plugin_basic_datas.get("plugin-type", "unknown"),
                )
                Print.print_inf(
                    f" {i + 1}. §e{plugin_name} §av{plugin_basic_datas['version']} "
                    f"§b@{plugin_basic_datas['author']} §d{plugin_type}插件",
                    need_log=False,
                )
            Print.print_inf(
                f"§f第 {start_index // CTXS + 1} / {total_pages} 页，输入 §b+§f/§b- §f翻页",
                need_log=False,
            )
            Print.print_inf("§f输入插件序号选中插件并查看其下载页", need_log=False)

        def handle_plugin_selection(plugin_data: PluginRegData):
            """处理插件选择

            Args:
                plugin_data (dict): 插件数据

            Returns:
                bool: 是否退出
            """
            ok, pres = self.choice_plugin(plugin_data)
            if ok:
                if in_game and plugin_data.plugin_id not in plugin_group.loaded_plugin_ids:
                    resp = (
                        input(
                            Print.fmt_info(
                                f"§f可以直接热加载该插件: {plugin_data.name}, 是否加载(§aY§f/§cN§f): "
                            )
                        )
                        .strip()
                        .lower()
                    )
                    if resp == "y":
                        for i in pres:
                            if i.plugin_id not in plugin_group.loaded_plugin_ids:
                                try:
                                    plugin_group.load_plugin_hot(i.name, i.plugin_type)
                                except Exception as err:
                                    input(Print.fmt_info(f"插件热加载出现问题：{err}", "§c 报错 §r"))
                else:
                    Print.print_inf(
                        "插件已存在，若要更新版本，请重启 ToolDelta", need_log=False
                    )
                return (
                    input(
                        Print.fmt_info("§f输入 §cq §f退出, 其他则返回插件市场")
                    ).lower()
                    == "q"
                )
            else:
                Print.print_inf("已取消。", need_log=False)
                time.sleep(1)
            return False

        try:
            market_datas = self.get_datas_from_market(source_url or "")
            plugin_ids_map = self.plugin_id_name_map
            plugins_list = list(market_datas["MarketPlugins"].items())
            all_indexes = len(plugins_list)
            now_index = 0
            sum_pages = (all_indexes - 1) // CTXS + 1
            last_operation = ""

            while True:
                display_plugins(now_index, sum_pages)

                last_operation = (
                    input(
                        Print.fmt_info(
                            "§f回车键继续上次操作，§bq§f 退出，请输入：", "§f 输入 "
                        )
                    )
                    or last_operation
                )
                last_operation = last_operation.lower().strip()

                # 翻页操作
                if last_operation in ["+", "-"]:
                    now_index = max(
                        0,
                        min(
                            now_index + (CTXS if last_operation == "+" else -CTXS),
                            all_indexes - 1,
                        ),
                    )
                elif last_operation == "q":  # 退出操作
                    break
                else:
                    res = Utils.try_int(last_operation)
                    if res and 1 <= res <= all_indexes:  # 插件选择操作
                        plugin_data = self.get_plugin_data_from_market(
                            plugins_list[res - 1][0]
                        )
                        if handle_plugin_selection(plugin_data):
                            break
                    else:  # 超出序号范围
                        Print.print_err("超出序号范围")

        except (KeyError, requests.RequestException) as err:
            Print.print_err(f"获取插件市场插件出现问题：{err}")
        except Exception:
            Print.print_err("获取插件市场插件出现问题，报错如下：")
            Print.print_err(traceback.format_exc())
        finally:
            clear_screen()
            Print.clean_print("§a已从插件市场返回 ToolDelta 控制台。")

    def get_datas_from_market(self, source_url: str | None = None) -> dict:
        """
        从插件市场的 market_tree.json 获取数据

        Args:
            source_url (str | None, optional): 插件市场源，默认为 None, 有则替换整体插件市场源

        Returns:
            dict: 插件市场数据
        """
        if isinstance(source_url, str):
            self.plugins_download_url = self.plugins_download_url
        market_datas = get_json_from_url(
            url_join(self.plugins_download_url, "market_tree.json")
        )
        return market_datas

    def get_plugin_data_from_market(self, plugin_id: str) -> PluginRegData:
        """从插件市场获取单个插件数据

        Args:
            plugin_id (str): 插件 ID

        Raises:
            KeyError: 无法通过 ID 查找插件

        Returns:
            PluginRegData: 插件注册数据
        """
        if self.plugin_id_name_map is None:
            self.plugin_id_name_map = self.get_plugin_id_name_map()
        plugin_name = self.plugin_id_name_map.get(plugin_id)
        if plugin_name is None:
            raise requests.RequestException(
                f"无法通过 ID: {plugin_id} 查找插件, 你可能需要反馈此问题至开发者"
            )
        data_url = self.plugins_download_url + "/" + plugin_name + "/datas.json"
        datas = get_json_from_url(data_url)
        return PluginRegData(plugin_name, datas)

    def choice_plugin(
        self, plugin_data: PluginRegData
    ) -> tuple[bool, list[PluginRegData]]:
        """选中插件进行介绍与操作

        Args:
            plugin_data (PluginRegData): 插件注册数据

        Returns:
            tuple[bool, list[PluginRegData]]: 是否下载，下载的插件列表
        """
        pre_plugins_str = (
            ", ".join([f"{k}§7v{v}" for k, v in plugin_data.pre_plugins.items()])
            or "无"
        )
        clear_screen()
        Print.print_inf(
            f"{plugin_data.name} v{plugin_data.version_str}", need_log=False
        )
        Print.print_inf(
            f"作者：§f{plugin_data.author}§7, 版本：§f{plugin_data.version_str} §b{plugin_data.plugin_type_str}",
            need_log=False,
        )
        Print.print_inf(f"前置插件：§f{pre_plugins_str}", need_log=False)
        Print.print_inf(f"介绍：{plugin_data.description}", need_log=False)
        Print.print_inf("", need_log=False)
        res = (
            input(Print.fmt_info("§f下载 = §aY§f, 取消 = §cN§f, 请输入："))
            .lower()
            .strip()
        )
        if res == "y":
            Print.clean_print(f"§6正在下载插件：§f{plugin_data.name}", end="\r")
            pres = self.download_plugin(plugin_data)
            pres.reverse()
            return True, pres
        return False, []

    def get_plugin_id_name_map(self) -> dict:
        """获取插件 ID 与插件名的映射

        Returns:
            dict: 插件 ID 与插件名的映射
        """
        res = requests.get(
            self.plugins_download_url + "/plugin_ids_map.json", timeout=5
        )
        res.raise_for_status()
        res1: dict = json.loads(res.text)
        self.plugin_id_name_map = res1
        return res1

    def get_download_list(self, plugin_data: PluginRegData) -> dict[str, PluginRegData]:
        """
        获取一个插件的下载列表

        Args:
            plugin_data (PluginRegData): 插件注册数据

        Returns:
            list[str]: 下载列表
        """
        if self.plugin_id_name_map is None:
            self.plugin_id_name_map = self.get_plugin_id_name_map()
        download_paths = {}
        stack = [plugin_data]
        while stack:
            current_plugin = stack.pop()
            download_paths[current_plugin.name] = current_plugin
            for plugin_id in current_plugin.pre_plugins:
                plugin_datas = self.get_plugin_data_from_market(plugin_id)
                stack.append(plugin_datas)
        return download_paths

    def download_plugin(
        self, plugin_data: PluginRegData, with_pres=True, is_enabled=True
    ) -> list[PluginRegData]:
        """
        下载插件
        注意：只能传入由 `get_plugin_data_from_market()` 生成的数据

        Args:
            plugin_data (PluginRegData): 插件注册数据
            with_pres (bool): 是否一同下载前置插件
            is_enabled (bool): 下载的插件是否要自动禁用

        Raises:
            ValueError: 未知插件类型

        Returns:
            list[PluginRegData]: 本插件和其前置插件的注册数据列表
        """
        if self.plugin_id_name_map is None:
            self.plugin_id_name_map = self.get_plugin_id_name_map()
        # 打印正在获取的插件下载树
        Print.clean_print(
            f"§6正在获取插件下载树 §f{plugin_data.name}§6.." + " " * 15, end="\r"
        )
        plugin_list = self.get_download_list(plugin_data)
        plugin_filepaths_dict = {}
        for k, v in plugin_list.items():
            plugin_filepaths_dict[k] = self.find_dirs(v)
        Print.clean_print(f"§a成功获取插件下载树 §f{plugin_data.name}§a" + " " * 15)
        plugins_url2dst_solve: list[tuple[str, str]] = []
        for plugin_name, pluginInfo in plugin_list.items():
            match pluginInfo.plugin_type:
                case "classic":
                    plugintype_path = os.path.join("插件文件", TOOLDELTA_CLASSIC_PLUGIN)
                case "injected":
                    plugintype_path = os.path.join(
                        "插件文件", TOOLDELTA_INJECTED_PLUGIN
                    )
                case _:
                    raise ValueError(
                        f"未知插件类型：{pluginInfo.plugin_type}, 你可能需要通知 ToolDelta 项目开发组解决"
                    )
            for path in plugin_filepaths_dict[plugin_name]:
                plugins_url2dst_solve.append(
                    (
                        url_join(self.plugins_download_url, path),
                        os.path.join(plugintype_path, path),
                    )
                )
        Print.clean_print(
            f"§bTD下载管理器: §7需要下载 §c{len(plugins_url2dst_solve)} §7个文件"
        )
        asyncio.run(urlmethod.download_file_urls(plugins_url2dst_solve))
        Print.clean_print("§a• 插件安装已完成")
        return list(plugin_list.values())

    def find_dirs(self, plugin_data: PluginRegData) -> list[str]:
        """
        查找插件目录

        Args:
            plugin_data (PluginRegData): 插件注册数据

        Raises:
            KeyError: 目录结构错误
            Exception: 获取插件市场插件目录结构出现问题

        Returns:
            list[str]: 插件目录列表
        """
        try:
            data = get_json_from_url(
                url_join(self.plugins_download_url, "directory.json")
            )
            data_list = []
            for folder, files in data.items():
                if plugin_data.name == folder.split(r"/")[0]:
                    # 展开
                    for file in files:
                        data_list.append(folder + r"/" + file)
            return data_list
        except KeyError as err:
            Print.print_err(
                f"获取插件市场插件目录结构出现问题：无法找到 {err}, 有可能是未来得及更新目录"
            )
            raise KeyError(f"无法找到 {err}, 有可能是未来得及更新目录") from err
        except Exception as err:
            Print.print_err(f"获取插件市场插件目录结构出现问题：{err}")
            raise KeyError(f"获取插件市场插件目录结构出现问题：{err}") from err

    def get_latest_plugin_version(self, plugin_id: str) -> str:
        """获取最新插件版本

        Args:
            plugin_id (str): 插件 ID

        Raises:
            KeyError: 无法通过 ID 获取最新插件版本

        Returns:
            str: 最新插件版本
        """
        result = get_json_from_url(
            url_join(self.plugins_download_url, "latest_versions.json")
        ).get(plugin_id)
        if isinstance(result, str):
            return result
        raise KeyError(f"无法通过 ID: {plugin_id} 获取最新插件版本")


market = PluginMarket()



"ToolDelta 类式插件"

import importlib
import os
import sys
import traceback
from typing import TYPE_CHECKING, TypeVar

from ...cfg import Cfg
from ...color_print import Print
from ...constants import TOOLDELTA_CLASSIC_PLUGIN, TOOLDELTA_PLUGIN_DATA_DIR
from ...plugin_load import NotValidPluginError, plugin_is_enabled
from ...utils import Utils

if TYPE_CHECKING:
    # 类型注释
    from ... import ToolDelta
    from ...plugin_load.PluginGroup import PluginGroup

__caches__ = {"plugin": None, "api_name": "", "frame": None}


class Plugin:
    "插件主类"

    name: str = ""
    "插件名"
    version = (0, 0, 1)
    "插件版本号"
    author = "?"
    "作者名"
    description = "..."
    "简介"

    __path_created__ = False

    def __init__(self, frame: "ToolDelta"):
        self.frame = frame
        self.game_ctrl = frame.get_game_control()

    @property
    def data_path(self) -> str:
        "该插件的数据文件夹路径 (调用时直接创建数据文件夹)"
        path = os.path.join(TOOLDELTA_PLUGIN_DATA_DIR, self.name)
        if not self.__path_created__:
            os.makedirs(path, exist_ok=True)
            self.__path_created__ = True
        return path

    def make_data_path(self):
        os.makedirs(os.path.join(TOOLDELTA_PLUGIN_DATA_DIR, self.name), exist_ok=True)


_PLUGIN_CLS_TYPE = TypeVar("_PLUGIN_CLS_TYPE")


def add_plugin(plugin: type[_PLUGIN_CLS_TYPE]) -> type[_PLUGIN_CLS_TYPE]:
    try:
        if not Plugin.__subclasscheck__(plugin):
            raise NotValidPluginError(f"插件主类必须继承 Plugin 类 而不是 {plugin}")
    except TypeError as exc:
        raise NotValidPluginError(
            f"插件主类必须继承 Plugin 类 而不是 {plugin.__class__}"
        ) from exc
    if __caches__["plugin"] is not None:
        raise NotValidPluginError("调用了多次 @add_plugin")
    if __caches__["frame"] is None:
        Print.clean_print("§d正在以直接运行模式运行插件..")
        return plugin
    plugin_ins = plugin(__caches__["frame"])  # type: ignore
    __caches__["plugin"] = plugin_ins
    return plugin

def add_plugin_as_api(apiName: str):
    def _add_plugin_2_api(api_plugin: type[_PLUGIN_CLS_TYPE]) -> type[_PLUGIN_CLS_TYPE]:
        if not Plugin.__subclasscheck__(api_plugin):
            raise NotValidPluginError("API 插件主类必须继承 Plugin 类")
        if __caches__["plugin"] is not None:
            raise NotValidPluginError("调用了多次 @add_plugin")
        if __caches__["frame"] is None:
            Print.clean_print("§d正在以直接运行模式运行插件..")
            return api_plugin
        plugin_ins = api_plugin(__caches__["frame"])  # type: ignore
        __caches__["plugin"] = plugin_ins
        __caches__["api_name"] = apiName
        return api_plugin

    return _add_plugin_2_api


# Plugin get and execute


def read_plugins(plugin_grp: "PluginGroup") -> None:
    """读取插件

    Args:
        plugin_grp (PluginGroup): 插件组
    """
    PLUGIN_PATH = os.path.join("插件文件", TOOLDELTA_CLASSIC_PLUGIN)
    sys.path.append(os.path.join("插件文件", TOOLDELTA_CLASSIC_PLUGIN))
    for plugin_dir in os.listdir(PLUGIN_PATH):
        if not plugin_is_enabled(plugin_dir):
            continue
        if os.path.isdir(os.path.join(PLUGIN_PATH, plugin_dir)):
            sys.path.append(os.path.join(PLUGIN_PATH, plugin_dir))
            load_plugin(plugin_grp, plugin_dir)
            if os.path.isfile(data_path := os.path.join("插件文件", TOOLDELTA_CLASSIC_PLUGIN, plugin_dir, "datas.json")):
                plugin_data = Utils.JsonIO.SafeJsonLoad(data_path)
                plugin_grp.loaded_plugin_ids.append(plugin_data["plugin-id"])


def load_plugin(
    plugin_group: "PluginGroup", plugin_dirname: str
) -> None | Plugin:
    """加载插件

    Args:
        plugin_group (PluginGroup): 插件组类
        plugin_dirname (str): 插件目录名
        hot_load (bool, optional): 是否热加载

    Raises:
        ValueError: 插件组未初始化读取
        ValueError: 插件组未绑定框架
        ValueError: 插件主类需要作者名
        ValueError: 插件主类需要作者名
        NotValidPluginError: 插件 不合法
        SystemExit: 插件名字不合法
        SystemExit: 插件配置文件报错
        SystemExit: 插件读取数据失败

    Returns:
        Union[None, Plugin]: 插件实例
    """
    if isinstance(plugin_group, type(None)):
        raise ValueError("插件组未初始化读取")
    if isinstance(plugin_group.linked_frame, type(None)):
        raise ValueError("插件组未绑定框架")
    __caches__["plugin"] = None
    __caches__["api_name"] = ""
    try:
        if os.path.isfile(
            os.path.join(
                "插件文件", TOOLDELTA_CLASSIC_PLUGIN, plugin_dirname, "__init__.py"
            )
        ):
            importlib.import_module(plugin_dirname)
        else:
            Print.print_war(f"{plugin_dirname} 文件夹 未发现插件文件，跳过加载")
            return None
        plugin_or_none: Plugin | None = __caches__.get("plugin")
        if plugin_or_none is None:
            raise NotValidPluginError(
                "需要调用 1 次 @plugins.add_plugin 以注册插件主类，然而没有调用"
            )
        plugin: Plugin = plugin_or_none
        if plugin.name is None or plugin.name == "":
            raise ValueError(f"插件主类 {plugin.__class__.__name__} 需要作者名")
        plugin_group.plugins.append(plugin)
        _v0, _v1, _v2 = plugin.version
        for evt_name in (
            "on_def",
            "on_inject",
            "on_player_prejoin",
            "on_player_join",
            "on_player_message",
            "on_player_death",
            "on_player_leave",
            "on_command",
            "on_frame_exit",
        ):
            if hasattr(plugin, evt_name):
                plugin_group.plugins_funcs[evt_name].append(
                    [plugin.name, getattr(plugin, evt_name)]
                )
        Print.print_suc(
            f"成功载入插件 {plugin.name} 版本：{_v0}.{_v1}.{_v2} 作者：{plugin.author}"
        )
        plugin_group.normal_plugin_loaded_num += 1
        if plugin_group.plugin_added_cache["packets"] != []:
            for pktType, func in plugin_group.plugin_added_cache["packets"]:
                ins_func = getattr(plugin, func.__name__)
                if ins_func is None:
                    raise NotValidPluginError("数据包监听不能在主插件类以外定义")
                plugin_group.add_listen_packet_id(pktType)
                plugin_group.add_listen_packet_func(pktType, ins_func)
        if __caches__["api_name"] != "":
            plugin_group.plugins_api[__caches__["api_name"]] = plugin
        if plugin_group.broadcast_evts_cache != {}:
            for evt, funcs in plugin_group.broadcast_evts_cache.items():
                for func in funcs:
                    ins_func = getattr(plugin, func.__name__)
                    if ins_func is None:
                        raise NotValidPluginError("广播事件监听不能在主插件类以外定义")
                    plugin_group.add_broadcast_evt(evt, ins_func)
        return plugin
    except NotValidPluginError as err:
        Print.print_err(f"插件 {plugin_dirname} 不合法：{err.args[0]}")
        raise SystemExit from err
    except Cfg.ConfigError as err:
        Print.print_err(f"插件 {plugin_dirname} 配置文件报错：{err}")
        Print.print_err(
            "你也可以直接删除配置文件，重新启动 ToolDelta 以自动生成配置文件"
        )
        raise SystemExit from err
    except Utils.SimpleJsonDataReader.DataReadError as err:
        Print.print_err(f"插件 {plugin_dirname} 读取数据失败：{err}")
    except plugin_group.linked_frame.SystemVersionException as err:
        Print.print_err(f"插件 {plugin_dirname} 需要更高版本的 ToolDelta 加载：{err}")
        raise SystemExit
    except Exception as err:
        Print.print_err(f"加载插件 {plugin_dirname} 出现问题，报错如下：")
        Print.print_err("§c" + traceback.format_exc())
        raise SystemExit from err
    finally:
        plugin_group.plugin_added_cache["packets"].clear()
        plugin_group.broadcast_evts_cache.clear()
    return None


def _init_frame(frame: "ToolDelta"):
    __caches__["frame"] = frame



"获取 ToolDelta 的版本号"


def get_tool_delta_version() -> tuple[int, ...]:
    """获取 ToolDelta 的版本号

    Returns:
        tuple[int, ...]: 版本号
    """
    try:
        with open("version", encoding="utf-8") as file:
            return tuple(int(v) for v in file.read().strip().split("."))
    except Exception:
        # 该返回值不必手动修改，
        # 它会由 GitHub Action 自动从 version 文件同步
        return (0, 7, 28)



"""
游戏交互实用方法
"""

import time
from typing import TYPE_CHECKING, Optional

import ujson as json

from tooldelta.color_print import Print
from tooldelta.constants import PacketIDS

from .packets import Packet_CommandOutput
from .utils import Utils

if TYPE_CHECKING:
    from tooldelta import GameCtrl, ToolDelta

game_ctrl: Optional["GameCtrl"] = None
movent_frame: Optional["ToolDelta"] = None


def _set_frame(my_Frame: "ToolDelta") -> None:
    """
    全局初始化框架

    参数:
        my_Frame: 要设置的框架对象
    """
    global movent_frame, game_ctrl  # pylint: disable=global-statement
    movent_frame = my_Frame
    game_ctrl = my_Frame.get_game_control()


def _get_game_ctrl() -> "GameCtrl":
    """检查 GameCtrl 是否可用"""
    if isinstance(game_ctrl, type(None)):
        raise ValueError("GameControl 不可用")
    return game_ctrl


def _get_frame() -> "ToolDelta":
    """检查 GameCtrl 是否可用"""
    if isinstance(movent_frame, type(None)):
        raise ValueError("Frame 不可用")
    return movent_frame


def getTarget(sth: str, timeout: int = 5) -> list:
    """
    获取符合目标选择器实体的列表

    参数:
        sth: 目标选择器
        timeout: 超时时间，默认为 5 秒
    异常:
        ValueError: 指令返回超时，或者无法获取目标
    """
    game_ctrl = _get_game_ctrl()
    if not sth.startswith("@"):
        raise ValueError("我的世界目标选择器格式错误 (getTarget 必须使用目标选择器)")
    result = game_ctrl.sendcmd_with_resp(f"/testfor {sth}", timeout)
    if result.SuccessCount:
        result = result.OutputMessages[0].Parameters[0]
        return result.split(", ")
    if result.OutputMessages[0].Message == "commands.generic.syntax":
        raise ValueError(f"getTarget 目标选择器表达式错误：{sth}")
    return []


def getPos(target: str, timeout: float = 5) -> dict:
    """获取目标玩家的详细位置信息

    参数:
        targetNameToGet: 目标玩家的名称
        timeout: 超时时间（秒）。默认为 5 秒

    异常:
        ValueError: 当目标玩家不存在时抛出该异常
        ValueError: 当获取位置信息失败时抛出该异常
        AttributeError: 当获取玩家 UUID 失败时抛出该异常
    """
    game_ctrl = _get_game_ctrl()
    if (
        target not in game_ctrl.allplayers
        and not target.startswith("@")
        and target != game_ctrl.bot_name
    ):
        raise ValueError(f'玩家 "{target}" 不存在')
    resp = game_ctrl.sendcmd_with_resp(f'/querytarget @a[name="{target}"]', timeout)
    if not resp.OutputMessages[0].Success:
        raise ValueError(f"无法获取坐标信息：{resp.OutputMessages[0].Message}")
    parameter = json.loads(resp.OutputMessages[0].Parameters[0])
    if isinstance(parameter, str):
        raise ValueError("无法获取坐标信息：" + parameter)
    result = {}
    if game_ctrl.players_uuid is None:
        raise AttributeError("无法获取玩家 UUID 表")
    targetName = target
    x = (
        parameter[0]["position"]["x"]
        if parameter[0]["position"]["x"] >= 0
        else parameter[0]["position"]["x"] - 1
    )
    y = parameter[0]["position"]["y"] - 1.6200103759765
    z = (
        parameter[0]["position"]["z"]
        if parameter[0]["position"]["z"] >= 0
        else parameter[0]["position"]["z"] - 1
    )
    position = {
        "x": float(f"{x:.2f}"),
        "y": float(f"{y:.2f}"),
        "z": float(f"{z:.2f}"),
    }
    dimension = parameter[0]["dimension"]
    yRot = parameter[0]["yRot"]
    result[targetName] = {
        "dimension": dimension,
        "position": position,
        "yRot": yRot,
    }
    if target == "@a":
        return result
    if len(result) != 1:
        raise ValueError("获取坐标失败")
    if target.startswith("@a"):
        return next(iter(result.values()))
    return result[target]


def getItem(target: str, itemName: str, itemSpecialID: int = -1) -> int:
    """
    获取玩家背包内指定的物品的数量
    参数:
        targetName (str): 玩家选择器 / 玩家名
        itemName (str): 物品 ID
        itemSpecialID (int): 物品特殊值，默认值 -1
    """
    game_ctrl = _get_game_ctrl()
    if (
        (target not in game_ctrl.allplayers)
        and (target != game_ctrl.bot_name)
        and (not target.startswith("@a"))
    ):
        raise ValueError("未找到目标玩家")
    result: Packet_CommandOutput = game_ctrl.sendcmd_with_resp(
        f"/clear {target} {itemName} {itemSpecialID} 0"
    )
    if result.OutputMessages[0].Message == "commands.generic.syntax":
        raise ValueError("物品 ID 错误")
    if result.OutputMessages[0].Message == "commands.clear.failure.no.items":
        return 0
    return int(result.OutputMessages[0].Parameters[1])


def getPosXYZ(player, timeout: float = 30) -> tuple[float, float, float]:
    """
    获取玩家的简略坐标值，并以坐标三元元组返回
    参数:
        player (str): 玩家名
        timeout (int): 最长超时时间
    返回:
        tuple[float, float, float]
    """
    res = getPos(player, timeout=timeout)["position"]
    return res["x"], res["y"], res["z"]


def getMultiScore(scoreboardNameToGet: str, targetNameToGet: str) -> int | dict:
    """
    获取单个或多个计分板分数项
    参数:
        scoreboardNameToGet: 计分板名
        targetNameToGet: 获取分数的对象/目标选择器
    返回:
        分数：int
    异常:
        ValueError: 无法获取分数
    """
    game_ctrl = _get_game_ctrl()
    resultList = game_ctrl.sendcmd_with_resp(
        f"/scoreboard players list {targetNameToGet}"
    ).OutputMessages
    result = {}
    result2 = {}
    for i in resultList:
        Message = i.Message
        if Message == r"commands.scoreboard.players.list.player.empty":
            continue
        if Message == r"§a%commands.scoreboard.players.list.player.count":
            targetName = i.Parameters[1][1:]
        elif Message == "commands.scoreboard.players.list.player.entry":
            if targetName == "commands.scoreboard.players.offlinePlayerName":
                continue
            scoreboardName = i.Parameters[2]
            targetScore = int(i.Parameters[0])
            if targetName not in result:
                result[targetName] = {}
            result[targetName][scoreboardName] = targetScore
            if scoreboardName not in result2:
                result2[scoreboardName] = {}
            result2[scoreboardName][targetName] = targetScore
    if not (result or result2):
        raise Exception("获取计分板分数失败")
    try:
        if targetNameToGet == "*" or targetNameToGet.startswith("@"):
            return result2[scoreboardNameToGet]
        if scoreboardNameToGet == "*":
            return result[targetNameToGet]
        return result[targetNameToGet][scoreboardNameToGet]
    except KeyError as err:
        raise Exception(f"获取计分板分数失败：{err}")


def getScore(scb_name: str, target: str, timeout: float = 30) -> int:
    """获取计分板对应分数
    参数:
        scb_name: 计分板名
        target: 目标选择器
        timeout: 超时时间

    异常:
        ValueError: 计分板错误

    返回:
        int: 计分板分数
    """
    game_ctrl = _get_game_ctrl()
    if target == "*" or scb_name == "*":
        raise ValueError("在此处无法使用 通配符 作为计分板分数获取目标")
    resp = game_ctrl.sendcmd_with_resp(
        f"/scoreboard players test {target} {scb_name} 0 0", timeout
    ).OutputMessages[0]
    if resp.Message == "commands.scoreboard.objectiveNotFound":
        raise ValueError(f"计分板 {scb_name} 未找到")
    if resp.Message == "commands.scoreboard.players.list.player.empty":
        raise ValueError(f"计分板项或玩家 {target} 未找到")
    if resp.Message == "commands.scoreboard.players.score.notFound":
        raise ValueError(f"计分板项或玩家 {target} 在此计分板没有分数")
    if len(resp.Parameters) < 1:
        raise ValueError(
            f"计分板分数获取的Parameters获取异常: {resp.Message}: {resp.Parameters}"
        )
    return int(resp.Parameters[0])


def isCmdSuccess(cmd: str, timeout: float = 30):
    """
    获取命令执行成功与否的状态
    参数:
        cmd: MC 指令
        timeout: 超时时间
    返回:
        命令执行是否成功：bool
    """
    game_ctrl = _get_game_ctrl()
    res = game_ctrl.sendcmd_with_resp(cmd, timeout).SuccessCount
    return bool(res)


def sendcmd(
    cmd: str, waitForResp: bool = False, timeout: int = 30
) -> None | Packet_CommandOutput:
    r"""发送命令到游戏控制器，并可选择是否等待响应

    如果 waitForResp 为 False，则返回 None，否则返回 Packet_CommandOutput 对象

    参数:
        cmd: 要发送的命令
        waitForResp: 是否等待响应，默认为 False
        timeout: 等待响应的超时时间（秒）,默认为 30
    """
    game_ctrl = _get_game_ctrl()
    return game_ctrl.sendcmd(cmd, waitForResp, timeout)


def sendwscmd(
    cmd: str, waitForResp: bool = False, timeout: float = 30
) -> Packet_CommandOutput | None:
    """
    发送 WSCMD 命令到游戏控制器

    参数:
        cmd: 要发送的 WSCMD 命令
        waitForResp: 是否等待响应 默认为 False
        timeout: 超时时间（秒）默认为 30
    """
    game_ctrl = _get_game_ctrl()
    return game_ctrl.sendwscmd(cmd, waitForResp, timeout)


def sendwocmd(cmd: str) -> None:
    """
    发送 WO 命令到游戏控制器

    参数:
        cmd: 要发送的 WO 命令
    """
    game_ctrl = _get_game_ctrl()
    game_ctrl.sendwocmd(cmd)


def sendPacket(pktID: int, pkt: dict) -> None:
    """
    发送数据包给游戏控制器

    参数:
        pktID: 数据包 ID
        pkt: 数据包内容
    """
    game_ctrl = _get_game_ctrl()
    game_ctrl.sendPacket(pktID, pkt)


def rawText(playername: str, text: str) -> None:
    """
    向指定玩家发送原始文本消息

    参数:
        playername: 玩家名称
        text: 要发送的文本
    """
    game_ctrl = _get_game_ctrl()
    game_ctrl.say_to(playername, text)


def tellrawText(playername: str, title: str | None = None, text: str = "") -> None:
    """
    向指定玩家发送 tellraw 消息

    参数:
        playername: 玩家名称
        title: 标题文本（可选）
        text: 消息文本
    """
    game_ctrl = _get_game_ctrl()
    if title is None:
        game_ctrl.say_to(playername, text)
    else:
        game_ctrl.say_to(playername, f"<{title}§r> {text}")


def get_all_player() -> list:
    """获取所有玩家列表"""
    game_ctrl = _get_game_ctrl()
    return game_ctrl.allplayers


def is_op(playername: str) -> bool:
    """
    判断玩家是否为 OP

    参数:
        playername: 玩家名称
    """
    frame = _get_frame()
    return frame.launcher.is_op(playername)


def get_robotname() -> str:
    """获取机器人名称。"""
    game_ctrl = _get_game_ctrl()
    return game_ctrl.bot_name


def countdown(delay: float, msg: str | None = None) -> None:
    """
    倒计时函数

    参数:
        delay: 延迟时间，可以是整数或浮点数
        msg: 倒计时消息，可选参数，默认为"Countdown"
    """
    if msg is None:
        msg = "Countdown"
    delayStart = time.time()
    delayStop = delayStart + delay
    while delay >= 0:
        if delay >= 0:
            delay = delayStop - time.time()
        else:
            delay = 0
        Print.print_inf(f"{msg}: {delay:.2f}s", end="\r")
        time.sleep(0.01)
    print("", end="\n")


def getBlockTile(x: int, y: int, z: int) -> str:
    """
    获取指定坐标的方块的 ID

    参数:
        x: X 坐标
        y: Y 坐标
        z: Z 坐标
    """
    game_ctrl = _get_game_ctrl()
    res = game_ctrl.sendcmd_with_resp(f"/testforblock {x} {y} {z} air")
    if (
        res.SuccessCount
        or res.OutputMessages[0].Message == "commands.testforblock.outOfWorld"
    ):
        return "air"
    return res.OutputMessages[0].Parameters[4].strip("%tile.").strip(".name")


def getTickingAreaList() -> dict:
    """
    获取 tickingarea 列表

    异常:
        AttributeError: 获取 tickingarea 列表失败
    """
    result = {}
    cmd = sendwscmd("/tickingarea list all-dimensions", True)
    if cmd is not None:
        resultList = cmd.OutputMessages
    else:
        raise AttributeError("Failed to get the tickingarea list.")

    if resultList[0].Success is False:
        return result
    for tickareaData in resultList[1].Message.split("%dimension.dimensionName")[1:]:
        tickareaDimension = tickareaData.split(": \n")[0]
        tickareaList = tickareaData.split(": \n")[1].split("\n")
        for tickarea in tickareaList:
            if not tickarea:
                continue
            tickareaName = tickarea.split("- ", 1)[1].split(": ", 1)[0]
            tickareaPos = {
                "start": tickarea.split(" ")[2:5],
                "end": tickarea.split(" ")[6:9],
            }
            tickareaPos["start"].pop(1)
            tickareaPos["end"].pop(1)
            result[tickareaName] = {"dimension": tickareaDimension}
            result[tickareaName].update(tickareaPos)
    return result


def take_item_out_item_frame(pos: tuple[float, float, float]) -> None:
    """
    从物品展示框取出物品
    参数:
        position: 物品展示框的坐标 (x, y, z)
    返回:
        None
    """
    game_ctrl = _get_game_ctrl()
    BotPos: tuple[float, float, float] = getPosXYZ(game_ctrl.bot_name)
    game_ctrl.sendwocmd(
        f"tp {game_ctrl.bot_name} {str(int(pos[0])) + ' ' + str(int(pos[1])) + ' ' + str(int(pos[2]))}"
    )
    game_ctrl.sendPacket(PacketIDS.IDItemFrameDropItem, {"Position": pos})
    game_ctrl.sendwocmd(
        f"tp {game_ctrl.bot_name} {str(int(BotPos[0])) + ' ' + str(int(BotPos[1])) + ' ' + str(int(BotPos[2]))}"
    )


def __set_effect_while__(
    player_name: str, effect: str, level: int, particle: bool, icon_flicker: bool = True
) -> None:
    """
    内部方法: 设置玩家状态效果
    参数:
        player_name: 玩家名称 (String)
        effect: 效果 ID (String) 参考 EffectIDS 中内容
        level: 效果等级 (int) Max: 255
        particle: 是否显示粒子 (Boolean)
        icon_flicker: 是否使图标闪烁 (Boolean) [仅限ToolDelta运行时]
    返回:
        None
    """
    game_ctrl = _get_game_ctrl()
    command_prefix = f"/effect {player_name} {effect} "
    duration = "2" if icon_flicker else "1000000"
    command = f"{command_prefix}{duration} {level} {particle!s}"

    while True:
        result = game_ctrl.sendcmd_with_resp(command)
        if result.OutputMessages[0].Message == "commands.effect.success":
            time.sleep(1)
        else:
            break


def set_player_effect(
    player_name: str,
    effect: str,
    duration: int,
    level: int,
    particle: bool,
    icon_flicker: bool = True,
    timeout: float = 1.5,
) -> bool | ValueError:
    """
    设置玩家的状态效果

    参数:
        player_name: 玩家名称 (String) 或是选择器 (String)
        effect: 效果 ID (String) 参考 EffectIDS 中内容
        duration: 持续时间 (int) [为0代表永久(仅限ToolDelta运行时)] Max: 1000000
        level: 效果等级 (int) Max: 255
        particle: 是否显示粒子 (Boolean)
        icon_flicker: 是否使图标闪烁 (Boolean) [仅限ToolDelta运行时]
        timeout: 超时时间 (float) [可选]

    返回:
        Bool | ValueError: 是否设置成功
    """
    if level > 255:
        return ValueError(f"你提供的等级 ({level}) 太大了，它最高只能是 255。")  # type: ignore
    if duration > 1000000:
        return ValueError(
            f"你提供的持续时间 ({duration}) 太大了，它最高只能是 1000000。"
        )  # type: ignore

    game_ctrl = _get_game_ctrl()
    command = f"/effect {player_name} {effect} {duration} {level} {particle!s}"

    if duration == 0:
        Utils.createThread(
            func=__set_effect_while__,
            args=(player_name, effect, level, particle, icon_flicker),
            usage=f"Set_Effect_Thread_{player_name}",
        )
        return True

    result = game_ctrl.sendcmd_with_resp(command, timeout=timeout)
    match result.OutputMessages[0].Message:
        case "commands.generic.noTargetMatch":
            return ValueError(
                f"没有与选择器匹配的目标! 玩家 {player_name} 可能并不存在。"
            )  # type: ignore
        case "commands.effect.success":
            return True
        case _:
            return ValueError(f"未知错误: {result.OutputMessages[0].Message}")  # type: ignore



"支持 mc 颜色代码的输出模块"

import datetime
import threading
from typing import ClassVar

import colorama

from .logger import publicLogger

colorama.init(autoreset=True)


def simple_fmt(kw: dict, arg: str) -> str:
    """简单的字符串格式化

    Args:
        kw (dict): 颜色列表（key 为颜色代码，value 为颜色代码对应的颜色）
        arg (str): 需要格式化的字符串

    Returns:
        str: 格式化后的字符串
    """
    for k, v in kw.items():
        arg = arg.replace(k, str(v))
    return arg


class Print:
    """生成多样彩色输出的类"""

    INFO_NORMAL = "§f 信息 "
    INFO_WARN = "§6 警告 "
    INFO_ERROR = "§4 报错 "
    INFO_FAIL = "§c 失败 "
    INFO_SUCC = "§a 成功 "
    INFO_LOAD = "§d 加载 "
    STD_COLOR_LIST: ClassVar[list[list[str]]] = [
        ["0", "#000000"],
        ["1", "#0000AA"],
        ["2", "#00AA00"],
        ["3", "#00AAAA"],
        ["4", "#AA0000"],
        ["5", "#AA00AA"],
        ["6", "#FFAA00"],
        ["7", "#AAAAAA"],
        ["8", "#555555"],
        ["9", "#5555FF"],
        ["a", "#55FF55"],
        ["b", "#55FFFF"],
        ["c", "#FF5555"],
        ["d", "#FF55FF"],
        ["e", "#FFFF55"],
        ["f", "#FFFFFF"],
        ["g", "#DDD605"],
        ["r", "/"],
    ]

    lock = threading.RLock()

    @staticmethod
    def simple_fmt(kw: dict, arg: str) -> str:
        """简单的字符串格式化

        Args:
            kw (dict): 颜色列表（key 为颜色代码，value 为颜色代码对应的颜色）
            arg (str): 需要格式化的字符串

        Returns:
            str: 格式化后的字符串
        """
        return simple_fmt(kw, arg)

    @staticmethod
    def colormode_replace(text: str, showmode=0) -> str:
        """颜色代码替换

        Args:
            text (str): 需要替换的字符串
            showmode (int, optional): 显示模式

        Returns:
            str: 替换后的字符串
        """
        # 1 = bg_color
        text = Print._strike(text)
        return (
            simple_fmt(
                {
                    "§0": f"\033[{showmode};37;90m",
                    "§1": f"\033[{showmode};37;34m",
                    "§2": f"\033[{showmode};37;32m",
                    "§3": f"\033[{showmode};37;36m",
                    "§4": f"\033[{showmode};37;31m",
                    "§5": f"\033[{showmode};37;35m",
                    "§6": f"\033[{showmode};37;33m",
                    "§7": f"\033[{showmode};37;90m",
                    "§8": f"\033[{showmode};37;2m",
                    "§9": f"\033[{showmode};37;94m",
                    "§a": f"\033[{showmode};37;92m",
                    "§b": f"\033[{showmode};37;96m",
                    "§c": f"\033[{showmode};37;91m",
                    "§d": f"\033[{showmode};37;95m",
                    "§e": f"\033[{showmode};37;93m",
                    "§f": f"\033[{showmode};37;1m",
                    "§r": "\033[0m",
                    "§u": "\033[4m",
                    "§l": "\033[1m",
                },
                text,
            )
            + "\033[0m"
        )

    @staticmethod
    def align(text: str, length: int = 15) -> str:
        """对齐字符串

        Args:
            text (str): 需要对齐的字符串
            length (int, optional): 对齐长度

        Returns:
            str: 对齐后的字符串
        """
        text_length = len(text)
        for char in text:
            if not char.isascii():
                text_length += 1
        return text + " " * (length - text_length)

    @staticmethod
    def _strike(text: str) -> str:
        """删除线

        Args:
            text (str): 需要删除线的字符串

        Returns:
            str: 删除线后的字符串
        """
        text_ok = ""
        strike_mode = False
        i = 0
        while i < len(text):
            char = text[i]
            try:
                if char == "§":
                    if text[i + 1] == "s":
                        strike_mode = True
                        i += 2
                        continue
                    if text[i + 1] == "r":
                        strike_mode = False
            except IndexError:
                pass
            if strike_mode:
                text_ok += "\u0336" + char
            else:
                text_ok += char
            i += 1
        return text_ok

    @staticmethod
    def print_with_info(
        text: str, info: str = INFO_NORMAL, need_log: bool = True, **print_kwargs
    ):
        """输出带有信息的文本

        Args:
            text (str): 输出的文本
            info (str, optional): 输出的信息
            need_log (bool, optional): 是否需要记录日志
            **print_kwargs: 原 print 函数的参数

        Raises:
            AssertionError: 无法找到对应的颜色代码
        """
        with Print.lock:
            if need_log:
                Print.c_log(info, text)
            set_next_color = "§r"
            if "\n" in text:
                output_txts = []
                for text_line in str(text).split("\n"):
                    if "§" in text_line:
                        try:
                            n = text_line.rfind("§")
                            _set_next_col_value = text_line[n : n + 2]
                            if set_next_color == -1:
                                raise AssertionError
                            set_next_color = _set_next_col_value
                        except Exception:
                            pass
                    output_txts.append(
                        datetime.datetime.now().strftime("[%H:%M] ")
                        + Print.colormode_replace(info, 7)
                        + " "
                        + Print.colormode_replace(set_next_color + text_line)
                    )
                print("\n".join(output_txts), **print_kwargs)
            else:
                print(
                    datetime.datetime.now().strftime("[%H:%M] ")
                    + Print.colormode_replace(info, 7)
                    + " "
                    + Print.colormode_replace(text),
                    **print_kwargs,
                )

    @staticmethod
    def clean_print(text: str, **print_kwargs) -> None:
        """依照 mc 的颜色代码输出文本，可带有 print 函数的参数

        Args:
            text (str): 输出的文本
            **print_kwargs: 原 print 函数的参数
        """
        with Print.lock:
            print(Print.colormode_replace(text), **print_kwargs)

    @staticmethod
    def clean_fmt(text: str) -> str:
        """依照 mc 的颜色代码格式化文本

        Args:
            text (str): 需要格式化的文本

        Returns:
            str: 格式化后的文本
        """
        return Print.colormode_replace(text)

    @staticmethod
    def print(*args):
        Print.print_inf(" ".join(str(i) for i in args))

    @staticmethod
    def print_err(text: str, **print_kwargs) -> None:
        """输出错误信息

        Args:
            text (str): 输出的文本
        """
        Print.print_with_info(f"§c{text}", Print.INFO_ERROR, **print_kwargs)

    @staticmethod
    def print_inf(text: str, **print_kwargs) -> None:
        """输出 INDO 信息

        Args:
            text (str): 输出的文本
        """
        Print.print_with_info(f"{text}", Print.INFO_NORMAL, **print_kwargs)

    @staticmethod
    def print_suc(text: str, **print_kwargs) -> None:
        """输出成功信息

        Args:
            text (str): 输出的文本
        """
        Print.print_with_info(f"§a{text}", Print.INFO_SUCC, **print_kwargs)

    @staticmethod
    def print_war(text: str, **print_kwargs) -> None:
        """输出警告信息

        Args:
            text (str): 输出的文本
        """
        Print.print_with_info(f"§6{text}", Print.INFO_WARN, **print_kwargs)

    @staticmethod
    def print_load(text: str, **print_kwargs) -> None:
        """输出加载信息

        Args:
            text (str): 输出的文本
        """
        with Print.lock:
            Print.print_with_info(f"§d{text}", Print.INFO_LOAD, **print_kwargs)

    @staticmethod
    def fmt_info(text: str, info: str = "§f 信息 ") -> str:
        """格式化信息

        Args:
            text (str): 输出的文本
            info (str, optional): 输出的信息

        Raises:
            AssertionError: 无法找到对应的颜色代码

        Returns:
            str: 格式化后的信息
        """
        with Print.lock:
            nextcolor = "§r"
            if "\n" in text:
                output_txts = []
                for text_line in str(text).split("\n"):
                    if "§" in text_line:
                        try:
                            n = text_line.rfind("§")
                            _setnextcol = text_line[n : n + 2]
                            if nextcolor == -1:
                                raise AssertionError
                            nextcolor = _setnextcol
                        except Exception:
                            pass
                    output_txts.append(
                        datetime.datetime.now().strftime("[%H:%M] ")
                        + Print.colormode_replace(info, 7)
                        + " "
                        + Print.colormode_replace(nextcolor + text_line)
                    )
                return "\n".join(output_txts)
            return (
                datetime.datetime.now().strftime("[%H:%M] ")
                + Print.colormode_replace(info, 7)
                + " "
                + Print.colormode_replace(text)
            )

    @staticmethod
    def c_log(inf: str, msg: str) -> None:
        """记录日志

        Args:
            inf (str): 信息
            msg (str): 记录的信息
        """
        with Print.lock:
            for _g, _s in [
                ("§6 警告 ", "WARN"),
                ("§a 成功 ", "INFO"),
                ("§f 信息 ", "INFO"),
                ("§c 失败 ", "FAIL"),
                ("§4 报错 ", "ERROR"),
            ]:
                if inf == _g:
                    inf = _s
                    break
            for col, _ in Print.STD_COLOR_LIST:
                color_code = "§" + col
                msg = msg.replace(color_code, "")
            for col, _ in Print.STD_COLOR_LIST:
                color_code = "§" + col
                inf = inf.replace(color_code, "")
            inf = inf.replace(" ", "")
            publicLogger.log_in(msg, inf)
